package dev.inmo.postssystem.features.auth.client

import dev.inmo.micro_utils.common.Either
import dev.inmo.micro_utils.ktor.client.UnifiedRequester
import dev.inmo.postssystem.features.auth.common.AuthKey
import dev.inmo.postssystem.features.auth.common.AuthTokenInfo
import dev.inmo.postssystem.features.common.common.AdditionalModules
import dev.inmo.postssystem.features.status.client.StatusFeatureClient
import dev.inmo.postssystem.features.users.common.User
import io.ktor.client.HttpClient
import org.koin.core.module.Module
import org.koin.dsl.module

fun createAuthorizedFeaturesDIModule(
    serverUrl: String,
    initialAuthKey: Either<AuthKey, AuthTokenInfo>,
    onAuthKeyUpdated: suspend (AuthTokenInfo) -> Unit,
    onUserRetrieved: suspend (User?) -> Unit,
    onAuthKeyInvalidated: suspend () -> Unit
): Module {
    return module {
        single(AuthorizedQualifiers.CredsQualifier) { initialAuthKey }
        single(AuthorizedQualifiers.ServerUrlQualifier) { serverUrl }

        single (createdAtStart = true) {
            HttpClient {
                installClientAuthenticator(serverUrl, get(), get(AuthorizedQualifiers.CredsQualifier), onAuthKeyUpdated, onUserRetrieved, onAuthKeyInvalidated)
            }
        }
        single { UnifiedRequester(get(), get()) }
        single { StatusFeatureClient(get(AuthorizedQualifiers.ServerUrlQualifier), get()) }

        AdditionalModules.Authorized.modules.forEach {
            with(it) {
                load()
            }
        }
    }
}
